package model

import (
	"database/sql"
	"fmt"
	"strings"

	"git.i2edu.net/i2/go-zero/core/stores/sqlc"
	"git.i2edu.net/i2/go-zero/core/stores/sqlx"
	"git.i2edu.net/i2/go-zero/core/stringx"
	"git.i2edu.net/i2/go-zero/tools/goctl/model/sql/builderx"
)

var (
	baseOrganFieldNames          = builderx.RawFieldNames(&BaseOrgan{})
	baseOrganRows                = strings.Join(baseOrganFieldNames, ",")
	baseOrganRowsExpectAutoSet   = strings.Join(stringx.Remove(baseOrganFieldNames, "`id`", "`create_time`", "`update_time`"), ",")
	baseOrganRowsWithPlaceHolder = strings.Join(stringx.Remove(baseOrganFieldNames, "`id`", "`create_time`", "`update_time`"), "=?,") + "=?"
)

type (
	BaseOrganModel interface {
		Insert(data BaseOrgan) (sql.Result, error)
		FindOne(id int64) (*BaseOrgan, error)
		Update(data BaseOrgan) error
		Delete(id int64) error
	}

	defaultBaseOrganModel struct {
		conn  sqlx.SqlConn
		table string
	}

	BaseOrgan struct {
		LastUpdateTime sql.NullTime   `db:"last_update_time" json:"last_update_time"`
		OrganPinyin    sql.NullString `db:"organ_pinyin" json:"organ_pinyin"`
		CreateBy       sql.NullString `db:"create_by" json:"create_by"`
		Inheritance    sql.NullString `db:"inheritance" json:"inheritance"`
		Parent         sql.NullInt64  `db:"parent" json:"parent"`
		CreateTime     sql.NullTime   `db:"create_time" json:"create_time"`
		IfBuyOnline    sql.NullInt64  `db:"if_buy_online" json:"if_buy_online"`
		DelFlag        int64          `db:"del_flag" json:"del_flag"`
		ShengId        sql.NullInt64  `db:"sheng_id" json:"sheng_id"`
		Id             int64          `db:"id" json:"id"`
		OrganTell      sql.NullString `db:"organ_tell" json:"organ_tell"`
		OrganName      sql.NullString `db:"organ_name" json:"organ_name"`
		OrganType      sql.NullInt64  `db:"organ_type" json:"organ_type"`
		LastUpdateBy   sql.NullString `db:"last_update_by" json:"last_update_by"`
		OrganNumber    sql.NullString `db:"organ_number" json:"organ_number"`
		ParentId       sql.NullInt64  `db:"parent_id" json:"parent_id"`
		Name           sql.NullString `db:"name" json:"name"`
	}
)

func NewBaseOrganModel(conn sqlx.SqlConn) BaseOrganModel {
	return &defaultBaseOrganModel{
		conn:  conn,
		table: "`base_organ`",
	}
}

func (m *defaultBaseOrganModel) Insert(data BaseOrgan) (sql.Result, error) {
	query := fmt.Sprintf("insert into %s (%s) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", m.table, baseOrganRowsExpectAutoSet)
	ret, err := m.conn.Exec(query, data.LastUpdateTime, data.OrganPinyin, data.CreateBy, data.Inheritance, data.Parent, data.IfBuyOnline, data.DelFlag, data.ShengId, data.OrganTell, data.OrganName, data.OrganType, data.LastUpdateBy, data.OrganNumber, data.ParentId, data.Name)
	return ret, err
}

func (m *defaultBaseOrganModel) FindOne(id int64) (*BaseOrgan, error) {
	query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", baseOrganRows, m.table)
	var resp BaseOrgan
	err := m.conn.QueryRow(&resp, query, id)
	switch err {
	case nil:
		return &resp, nil
	case sqlc.ErrNotFound:
		return nil, ErrNotFound
	default:
		return nil, err
	}
}

func (m *defaultBaseOrganModel) Update(data BaseOrgan) error {
	query := fmt.Sprintf("update %s set %s where `id` = ?", m.table, baseOrganRowsWithPlaceHolder)
	_, err := m.conn.Exec(query, data.LastUpdateTime, data.OrganPinyin, data.CreateBy, data.Inheritance, data.Parent, data.IfBuyOnline, data.DelFlag, data.ShengId, data.OrganTell, data.OrganName, data.OrganType, data.LastUpdateBy, data.OrganNumber, data.ParentId, data.Name, data.Id)
	return err
}

func (m *defaultBaseOrganModel) Delete(id int64) error {
	query := fmt.Sprintf("delete from %s where `id` = ?", m.table)
	_, err := m.conn.Exec(query, id)
	return err
}
