package model

import (
	"database/sql"
	"fmt"
	"strings"

	"git.i2edu.net/i2/go-zero/core/stores/cache"
	"git.i2edu.net/i2/go-zero/core/stores/sqlc"
	"git.i2edu.net/i2/go-zero/core/stores/sqlx"
	"git.i2edu.net/i2/go-zero/core/stringx"
	"git.i2edu.net/i2/go-zero/tools/goctl/model/sql/builderx"
)

var (
	mktPartTimeUserFieldNames          = builderx.RawFieldNames(&MktPartTimeUser{})
	mktPartTimeUserRows                = strings.Join(mktPartTimeUserFieldNames, ",")
	mktPartTimeUserRowsExpectAutoSet   = strings.Join(stringx.Remove(mktPartTimeUserFieldNames, "`id`", "`create_time`", "`update_time`"), ",")
	mktPartTimeUserRowsWithPlaceHolder = strings.Join(stringx.Remove(mktPartTimeUserFieldNames, "`id`", "`create_time`", "`update_time`"), "=?,") + "=?"

	cacheMktPartTimeUserIdPrefix    = "cache:mktPartTimeUser:id:"
	cacheMktPartTimeUserPhonePrefix = "cache:mktPartTimeUser:phone:"
)

type (
	MktPartTimeUserModel interface {
		Insert(data MktPartTimeUser) (sql.Result, error)
		FindOne(id int64) (*MktPartTimeUser, error)
		FindOneByPhone(id string) (*MktPartTimeUser, error)
		Update(data MktPartTimeUser) error
		Delete(id int64) error
	}

	defaultMktPartTimeUserModel struct {
		sqlc.CachedConn
		table string
	}

	MktPartTimeUser struct {
		Id             int64          `db:"id"`
		Name           sql.NullString `db:"name"`
		CityName       sql.NullString `db:"city_name"`
		Sex            sql.NullInt64  `db:"sex"`
		CreateBy       sql.NullString `db:"create_by"`
		CreateTime     sql.NullTime   `db:"create_time"`
		LastUpdateBy   sql.NullString `db:"last_update_by"`
		LastUpdateTime sql.NullTime   `db:"last_update_time"`
		IponeNumber    sql.NullString `db:"ipone_number"`
		DelFlag        sql.NullInt64  `db:"del_flag"`
	}
)

func NewMktPartTimeUserModel(conn sqlx.SqlConn, c cache.CacheConf) MktPartTimeUserModel {
	return &defaultMktPartTimeUserModel{
		CachedConn: sqlc.NewConn(conn, c),
		table:      "`mkt_part_time_user`",
	}
}

func (m *defaultMktPartTimeUserModel) Insert(data MktPartTimeUser) (sql.Result, error) {
	query := fmt.Sprintf("insert into %s (%s) values (?, ?, ?, ?, ?, ?, ?)", m.table, mktPartTimeUserRowsExpectAutoSet)
	ret, err := m.ExecNoCache(query, data.Name, data.Sex, data.CreateBy, data.LastUpdateBy, data.LastUpdateTime, data.IponeNumber, data.DelFlag)

	return ret, err
}

func (m *defaultMktPartTimeUserModel) FindOne(id int64) (*MktPartTimeUser, error) {
	mktPartTimeUserIdKey := fmt.Sprintf("%s%v", cacheMktPartTimeUserIdPrefix, id)
	var resp MktPartTimeUser
	err := m.QueryRow(&resp, mktPartTimeUserIdKey, func(conn sqlx.SqlConn, v interface{}) error {
		query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", mktPartTimeUserRows, m.table)
		return conn.QueryRow(v, query, id)
	})
	switch err {
	case nil:
		return &resp, nil
	case sqlc.ErrNotFound:
		return nil, ErrNotFound
	default:
		return nil, err
	}
}

func (m *defaultMktPartTimeUserModel) FindOneByPhone(phone string) (*MktPartTimeUser, error) {
	mktPartTimeUserPhoneKey := fmt.Sprintf("%s%v", cacheMktPartTimeUserPhonePrefix, phone)
	var resp MktPartTimeUser
	err := m.QueryRow(&resp, mktPartTimeUserPhoneKey, func(conn sqlx.SqlConn, v interface{}) error {
		query := fmt.Sprintf("select %s from %s where `ipone_number` = ? limit 1", mktPartTimeUserRows, m.table)
		return conn.QueryRow(v, query, phone)
	})
	switch err {
	case nil:
		return &resp, nil
	case sqlc.ErrNotFound:
		return nil, ErrNotFound
	default:
		return nil, err
	}
}

func (m *defaultMktPartTimeUserModel) Update(data MktPartTimeUser) error {
	mktPartTimeUserIdKey := fmt.Sprintf("%s%v", cacheMktPartTimeUserIdPrefix, data.Id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("update %s set %s where `id` = ?", m.table, mktPartTimeUserRowsWithPlaceHolder)
		return conn.Exec(query, data.Name, data.Sex, data.CreateBy, data.LastUpdateBy, data.LastUpdateTime, data.IponeNumber, data.DelFlag, data.Id)
	}, mktPartTimeUserIdKey)
	return err
}

func (m *defaultMktPartTimeUserModel) Delete(id int64) error {

	mktPartTimeUserIdKey := fmt.Sprintf("%s%v", cacheMktPartTimeUserIdPrefix, id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("delete from %s where `id` = ?", m.table)
		return conn.Exec(query, id)
	}, mktPartTimeUserIdKey)
	return err
}

func (m *defaultMktPartTimeUserModel) formatPrimary(primary interface{}) string {
	return fmt.Sprintf("%s%v", cacheMktPartTimeUserIdPrefix, primary)
}

func (m *defaultMktPartTimeUserModel) queryPrimary(conn sqlx.SqlConn, v, primary interface{}) error {
	query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", mktPartTimeUserRows, m.table)
	return conn.QueryRow(v, query, primary)
}
