package model

import (
	"database/sql"
	"fmt"
	"strings"

	"git.i2edu.net/i2/go-zero/core/stores/cache"
	"git.i2edu.net/i2/go-zero/core/stores/sqlc"
	"git.i2edu.net/i2/go-zero/core/stores/sqlx"
	"git.i2edu.net/i2/go-zero/core/stringx"
	"git.i2edu.net/i2/go-zero/tools/goctl/model/sql/builderx"
)

var (
	mktResourceFieldNames          = builderx.RawFieldNames(&MktResource{})
	mktResourceRows                = strings.Join(mktResourceFieldNames, ",")
	mktResourceRowsExpectAutoSet   = strings.Join(stringx.Remove(mktResourceFieldNames, "`id`", "`update_time`"), ",")
	mktResourceRowsWithPlaceHolder = strings.Join(stringx.Remove(mktResourceFieldNames, "`id`", "`update_time`"), "=?,") + "=?"

	cacheMktResourceIdPrefix = "cache:mktResource:id:"
)

type (
	MktResourceModel interface {
		Insert(data MktResource) (sql.Result, error)
		FindOne(id int64) (*MktResource, error)
		Update(data MktResource) error
		Delete(id int64) error
	}

	defaultMktResourceModel struct {
		sqlc.CachedConn
		table string
	}

	MktResource struct {
		HfCs                   sql.NullFloat64 `db:"hf_cs"`       // 回访次数
		OrtherCard             sql.NullString  `db:"orther_card"` // 其他证件号码
		StuMydType             sql.NullString  `db:"stu_myd_type"`
		AgreementId            sql.NullInt64   `db:"agreement_id"`
		StuPhone2              sql.NullString  `db:"stu_phone2"` // 联系人2电话
		StuInvalidDate         sql.NullTime    `db:"stu_invalid_date"`
		OsName                 sql.NullString  `db:"os_name"` // 资源收集校区
		StuRemark              sql.NullString  `db:"stu_remark"`
		StuSex                 sql.NullInt64   `db:"stu_sex"` // 性别（数据字典）
		UserLoginState         sql.NullInt64   `db:"user_login_state"`
		StuEnName              sql.NullString  `db:"stu_en_name"` // 学员英文
		StuBir                 sql.NullTime    `db:"stu_bir"`     // 学员出生日
		SchId                  sql.NullInt64   `db:"sch_id"`      // 资源跟进校区ID
		LastUpdateBy           sql.NullString  `db:"last_update_by"`
		PtuserId               sql.NullInt64   `db:"ptuser_id"` // 兼职人员ID
		PkCurrentPay           sql.NullInt64   `db:"pk_current_pay"`
		RecipientInfo          sql.NullInt64   `db:"recipient_info"`
		TmkZxsDate             sql.NullTime    `db:"tmk_zxs_date"`
		StuLinkPerson          sql.NullString  `db:"stu_link_person"` // 联系人1
		StuLoadSchool          sql.NullInt64   `db:"stu_load_school"` // 资源收集校区
		LoadUser               sql.NullString  `db:"load_user"`       // 资源收集人
		StuDjMoney             sql.NullFloat64 `db:"stu_dj_money"`    // 定金金额
		Relation1              sql.NullInt64   `db:"relation1"`       // 联系人1关系（数据字典）
		QuaoYji                sql.NullInt64   `db:"quao_yji"`        // 渠道一级
		IdcardType             sql.NullInt64   `db:"idcard_type"`     // 证件号类型
		CrmId                  sql.NullInt64   `db:"crm_id"`
		MaType                 sql.NullInt64   `db:"ma_type"`          // 来源渠道（活动渠道数据字典）
		StuLinkPerson2         sql.NullString  `db:"stu_link_person2"` // 联系人2
		AttendSchool           sql.NullString  `db:"attend_school"`    // 就读学校
		TimerVisitTime         sql.NullTime    `db:"timer_visit_time"` // 定时回访日期
		TurnUserType           sql.NullString  `db:"turn_user_type"`
		StuNowJc               sql.NullInt64   `db:"stu_now_jc"`
		StuAddress             sql.NullString  `db:"stu_address"`           // 家庭住址
		RecommendStu           sql.NullInt64   `db:"recommend_stu"`         // 推荐学生ID
		StuSysState            int64           `db:"stu_sys_state"`         // 学员系统状态（数据字典）
		KhType                 sql.NullInt64   `db:"kh_type"`               // 客户类型（数据字典）
		StuFollowUpMonthly     sql.NullInt64   `db:"stu_follow_up_monthly"` // 新生跟进月状态（数据字典）
		ZzxsDate               sql.NullTime    `db:"zzxs_date"`
		UselessRemark          sql.NullString  `db:"useless_remark"`
		StuZcMoney             sql.NullFloat64 `db:"stu_zc_money"` // 暂存总金额
		Relation2              sql.NullInt64   `db:"relation2"`    // 联系人2关系（数据字典）
		Plfpbz                 sql.NullString  `db:"plfpbz"`
		HeadImage              sql.NullInt64   `db:"head_image"`
		StuAllotTmkdate        sql.NullTime    `db:"stu_allot_tmkdate"`
		Age                    sql.NullInt64   `db:"age"`           // 年龄
		StuPhone               sql.NullString  `db:"stu_phone"`     // 联系人1电话
		TjUser                 sql.NullString  `db:"tj_user"`       // 员工推荐人
		AgeGroup               sql.NullInt64   `db:"age_group"`     // 年龄段（数据字典）
		StuWxksDate            sql.NullTime    `db:"stu_wxks_date"` // 无效到期日期
		EmployeeReferralArea   sql.NullString  `db:"employee_referral_area"`
		Id                     int64           `db:"id"`
		CreateBy               sql.NullString  `db:"create_by"` // 资源录入人ID
		StuJifen               sql.NullFloat64 `db:"stu_jifen"` // 学员积分
		StuLevel               sql.NullInt64   `db:"stu_level"`
		TmkZxsUsername         sql.NullString  `db:"tmk_zxs_username"`
		StuType                int64           `db:"stu_type"`       // 学员类型(数据字典)
		NewVisitDate           sql.NullTime    `db:"new_visit_date"` // 最近回访时间
		StuCity                sql.NullString  `db:"stu_city"`       // 资源收集城市
		IsImportant            sql.NullInt64   `db:"is_important"`   // 是否重点(0否，1是）
		ParentsProfession      sql.NullString  `db:"parents_profession"`
		Remark                 sql.NullString  `db:"remark"`
		CreateTime             sql.NullTime    `db:"create_time"`
		DelFlag                int64           `db:"del_flag"`
		StuParentId            sql.NullInt64   `db:"stu_parent_id"`
		StuMydTypeMark         sql.NullString  `db:"stu_myd_type_mark"`
		StudentCustomerId      sql.NullString  `db:"student_customer_id"` // 跟进客服（新增字段）
		IsChildOfEmployee      sql.NullInt64   `db:"is_child_of_employee"`
		StuName                sql.NullString  `db:"stu_name"` // 学员姓名
		LastUpdateTime         sql.NullTime    `db:"last_update_time"`
		MaTypeRemark           sql.NullString  `db:"ma_type_remark"` // 来源备注
		Idcard                 sql.NullString  `db:"idcard"`         // 身份证号码
		Stuschcount            sql.NullFloat64 `db:"stuschcount"`
		DegreeDepositMoney     sql.NullFloat64 `db:"degree_deposit_money"` // 学位订金
		StuWxdqDate            sql.NullTime    `db:"stu_wxdq_date"`
		NewPlanDate            sql.NullTime    `db:"new_plan_date"`  // 计划回访日期
		StuSale                sql.NullString  `db:"stu_sale"`       // 销售
		StuCityId              sql.NullInt64   `db:"stu_city_id"`    // 资源收集城市ID
		CallType               sql.NullInt64   `db:"call_type"`      // 咨询类型(数据字典)
		Wechatid               sql.NullString  `db:"wechatid"`       // 微信ID
		NetworkDetail          sql.NullInt64   `db:"network_detail"` // 渠道细分（NETWORK_DETAIL表）
		NewAllotDate           sql.NullTime    `db:"new_allot_date"`
		EmployeeReferralDepart sql.NullString  `db:"employee_referral_depart"`
		TmkCity                sql.NullInt64   `db:"tmk_city"`
	}
)

func NewMktResourceModel(conn sqlx.SqlConn, c cache.CacheConf) MktResourceModel {
	return &defaultMktResourceModel{
		CachedConn: sqlc.NewConn(conn, c),
		table:      "`mkt_resource`",
	}
}

func (m *defaultMktResourceModel) Insert(data MktResource) (sql.Result, error) {
	query := fmt.Sprintf("insert into %s (%s) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", m.table, mktResourceRowsExpectAutoSet)
	ret, err := m.ExecNoCache(query, data.HfCs, data.OrtherCard, data.StuMydType, data.AgreementId, data.StuPhone2, data.StuInvalidDate, data.OsName, data.StuRemark, data.StuSex, data.UserLoginState, data.StuEnName, data.StuBir, data.SchId, data.LastUpdateBy, data.PtuserId, data.PkCurrentPay, data.RecipientInfo, data.TmkZxsDate, data.StuLinkPerson, data.StuLoadSchool, data.LoadUser, data.StuDjMoney, data.Relation1, data.QuaoYji, data.IdcardType, data.CrmId, data.MaType, data.StuLinkPerson2, data.AttendSchool, data.TimerVisitTime, data.TurnUserType, data.StuNowJc, data.StuAddress, data.RecommendStu, data.StuSysState, data.KhType, data.StuFollowUpMonthly, data.ZzxsDate, data.UselessRemark, data.StuZcMoney, data.Relation2, data.Plfpbz, data.HeadImage, data.StuAllotTmkdate, data.Age, data.StuPhone, data.TjUser, data.AgeGroup, data.StuWxksDate, data.EmployeeReferralArea, data.CreateBy, data.StuJifen, data.StuLevel, data.TmkZxsUsername, data.StuType, data.NewVisitDate, data.StuCity, data.IsImportant, data.ParentsProfession, data.Remark, data.CreateTime, data.DelFlag, data.StuParentId, data.StuMydTypeMark, data.StudentCustomerId, data.IsChildOfEmployee, data.StuName, data.LastUpdateTime, data.MaTypeRemark, data.Idcard, data.Stuschcount, data.DegreeDepositMoney, data.StuWxdqDate, data.NewPlanDate, data.StuSale, data.StuCityId, data.CallType, data.Wechatid, data.NetworkDetail, data.NewAllotDate, data.EmployeeReferralDepart, data.TmkCity)

	return ret, err
}

func (m *defaultMktResourceModel) FindOne(id int64) (*MktResource, error) {
	mktResourceIdKey := fmt.Sprintf("%s%v", cacheMktResourceIdPrefix, id)
	var resp MktResource
	err := m.QueryRow(&resp, mktResourceIdKey, func(conn sqlx.SqlConn, v interface{}) error {
		query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", mktResourceRows, m.table)
		return conn.QueryRow(v, query, id)
	})
	switch err {
	case nil:
		return &resp, nil
	case sqlc.ErrNotFound:
		return nil, ErrNotFound
	default:
		return nil, err
	}
}

func (m *defaultMktResourceModel) Update(data MktResource) error {
	mktResourceIdKey := fmt.Sprintf("%s%v", cacheMktResourceIdPrefix, data.Id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("update %s set %s where `id` = ?", m.table, mktResourceRowsWithPlaceHolder)
		return conn.Exec(query, data.HfCs, data.OrtherCard, data.StuMydType, data.AgreementId, data.StuPhone2, data.StuInvalidDate, data.OsName, data.StuRemark, data.StuSex, data.UserLoginState, data.StuEnName, data.StuBir, data.SchId, data.LastUpdateBy, data.PtuserId, data.PkCurrentPay, data.RecipientInfo, data.TmkZxsDate, data.StuLinkPerson, data.StuLoadSchool, data.LoadUser, data.StuDjMoney, data.Relation1, data.QuaoYji, data.IdcardType, data.CrmId, data.MaType, data.StuLinkPerson2, data.AttendSchool, data.TimerVisitTime, data.TurnUserType, data.StuNowJc, data.StuAddress, data.RecommendStu, data.StuSysState, data.KhType, data.StuFollowUpMonthly, data.ZzxsDate, data.UselessRemark, data.StuZcMoney, data.Relation2, data.Plfpbz, data.HeadImage, data.StuAllotTmkdate, data.Age, data.StuPhone, data.TjUser, data.AgeGroup, data.StuWxksDate, data.EmployeeReferralArea, data.CreateBy, data.StuJifen, data.StuLevel, data.TmkZxsUsername, data.StuType, data.NewVisitDate, data.StuCity, data.IsImportant, data.ParentsProfession, data.Remark, data.CreateTime, data.DelFlag, data.StuParentId, data.StuMydTypeMark, data.StudentCustomerId, data.IsChildOfEmployee, data.StuName, data.LastUpdateTime, data.MaTypeRemark, data.Idcard, data.Stuschcount, data.DegreeDepositMoney, data.StuWxdqDate, data.NewPlanDate, data.StuSale, data.StuCityId, data.CallType, data.Wechatid, data.NetworkDetail, data.NewAllotDate, data.EmployeeReferralDepart, data.TmkCity, data.Id)
	}, mktResourceIdKey)
	return err
}

func (m *defaultMktResourceModel) Delete(id int64) error {

	mktResourceIdKey := fmt.Sprintf("%s%v", cacheMktResourceIdPrefix, id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("delete from %s where `id` = ?", m.table)
		return conn.Exec(query, id)
	}, mktResourceIdKey)
	return err
}

func (m *defaultMktResourceModel) formatPrimary(primary interface{}) string {
	return fmt.Sprintf("%s%v", cacheMktResourceIdPrefix, primary)
}

func (m *defaultMktResourceModel) queryPrimary(conn sqlx.SqlConn, v, primary interface{}) error {
	query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", mktResourceRows, m.table)
	return conn.QueryRow(v, query, primary)
}
