package model

import (
	"database/sql"
	"fmt"
	"strings"

	"git.i2edu.net/i2/go-zero/core/stores/cache"
	"git.i2edu.net/i2/go-zero/core/stores/sqlc"
	"git.i2edu.net/i2/go-zero/core/stores/sqlx"
	"git.i2edu.net/i2/go-zero/core/stringx"
	"git.i2edu.net/i2/go-zero/tools/goctl/model/sql/builderx"
)

var (
	mktStudentMarketActivityFieldNames          = builderx.RawFieldNames(&MktStudentMarketActivity{})
	mktStudentMarketActivityRows                = strings.Join(mktStudentMarketActivityFieldNames, ",")
	mktStudentMarketActivityRowsExpectAutoSet   = strings.Join(stringx.Remove(mktStudentMarketActivityFieldNames, "`id`", "`create_time`", "`update_time`"), ",")
	mktStudentMarketActivityRowsWithPlaceHolder = strings.Join(stringx.Remove(mktStudentMarketActivityFieldNames, "`id`", "`create_time`", "`update_time`"), "=?,") + "=?"

	cacheMktStudentMarketActivityIdPrefix = "cache:mktStudentMarketActivity:id:"
)

type (
	MktStudentMarketActivityModel interface {
		Insert(data MktStudentMarketActivity) (sql.Result, error)
		FindOne(id int64) (*MktStudentMarketActivity, error)
		Update(data MktStudentMarketActivity) error
		Delete(id int64) error
	}

	defaultMktStudentMarketActivityModel struct {
		sqlc.CachedConn
		table string
	}

	MktStudentMarketActivity struct {
		Id             int64          `db:"id"`
		SutId          sql.NullInt64  `db:"sut_id"`
		MaId           sql.NullInt64  `db:"ma_id"`
		CreateBy       sql.NullString `db:"create_by"`
		CreateTime     sql.NullTime   `db:"create_time"`
		LastUpdateBy   sql.NullString `db:"last_update_by"`
		LastUpdateTime sql.NullTime   `db:"last_update_time"`
		DelFlag        sql.NullInt64  `db:"del_flag"`
	}
)

func NewMktStudentMarketActivityModel(conn sqlx.SqlConn, c cache.CacheConf) MktStudentMarketActivityModel {
	return &defaultMktStudentMarketActivityModel{
		CachedConn: sqlc.NewConn(conn, c),
		table:      "`mkt_student_market_activity`",
	}
}

func (m *defaultMktStudentMarketActivityModel) Insert(data MktStudentMarketActivity) (sql.Result, error) {
	query := fmt.Sprintf("insert into %s (%s) values (?, ?, ?, ?, ?, ?)", m.table, mktStudentMarketActivityRowsExpectAutoSet)
	ret, err := m.ExecNoCache(query, data.SutId, data.MaId, data.CreateBy, data.LastUpdateBy, data.LastUpdateTime, data.DelFlag)

	return ret, err
}

func (m *defaultMktStudentMarketActivityModel) FindOne(id int64) (*MktStudentMarketActivity, error) {
	mktStudentMarketActivityIdKey := fmt.Sprintf("%s%v", cacheMktStudentMarketActivityIdPrefix, id)
	var resp MktStudentMarketActivity
	err := m.QueryRow(&resp, mktStudentMarketActivityIdKey, func(conn sqlx.SqlConn, v interface{}) error {
		query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", mktStudentMarketActivityRows, m.table)
		return conn.QueryRow(v, query, id)
	})
	switch err {
	case nil:
		return &resp, nil
	case sqlc.ErrNotFound:
		return nil, ErrNotFound
	default:
		return nil, err
	}
}

func (m *defaultMktStudentMarketActivityModel) Update(data MktStudentMarketActivity) error {
	mktStudentMarketActivityIdKey := fmt.Sprintf("%s%v", cacheMktStudentMarketActivityIdPrefix, data.Id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("update %s set %s where `id` = ?", m.table, mktStudentMarketActivityRowsWithPlaceHolder)
		return conn.Exec(query, data.SutId, data.MaId, data.CreateBy, data.LastUpdateBy, data.LastUpdateTime, data.DelFlag, data.Id)
	}, mktStudentMarketActivityIdKey)
	return err
}

func (m *defaultMktStudentMarketActivityModel) Delete(id int64) error {

	mktStudentMarketActivityIdKey := fmt.Sprintf("%s%v", cacheMktStudentMarketActivityIdPrefix, id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("delete from %s where `id` = ?", m.table)
		return conn.Exec(query, id)
	}, mktStudentMarketActivityIdKey)
	return err
}

func (m *defaultMktStudentMarketActivityModel) formatPrimary(primary interface{}) string {
	return fmt.Sprintf("%s%v", cacheMktStudentMarketActivityIdPrefix, primary)
}

func (m *defaultMktStudentMarketActivityModel) queryPrimary(conn sqlx.SqlConn, v, primary interface{}) error {
	query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", mktStudentMarketActivityRows, m.table)
	return conn.QueryRow(v, query, primary)
}
