package model

import (
	"database/sql"
	"fmt"
	"strings"

	"git.i2edu.net/i2/go-zero/core/stores/cache"
	"git.i2edu.net/i2/go-zero/core/stores/sqlc"
	"git.i2edu.net/i2/go-zero/core/stores/sqlx"
	"git.i2edu.net/i2/go-zero/core/stringx"
	"git.i2edu.net/i2/go-zero/tools/goctl/model/sql/builderx"
)

var (
	studentFieldNames          = builderx.RawFieldNames(&Student{})
	studentRows                = strings.Join(studentFieldNames, ",")
	studentRowsExpectAutoSet   = strings.Join(stringx.Remove(studentFieldNames, "`create_time`", "`update_time`"), ",")
	studentRowsWithPlaceHolder = strings.Join(stringx.Remove(studentFieldNames, "`id`", "`create_time`", "`update_time`"), "=?,") + "=?"

	cacheStudentIdPrefix = "cache#student#id#"
)

type (
	StudentModel interface {
		Insert(data Student) (sql.Result, error)
		FindOne(id int64) (*Student, error)
		Update(data Student) error
		Delete(id int64) error
	}

	defaultStudentModel struct {
		sqlc.CachedConn
		table string
	}

	Student struct {
		StuSex                 sql.NullInt64   `db:"stu_sex"` // 性别（数据字典）
		DelFlag                int64           `db:"del_flag"`
		Wechatid               sql.NullString  `db:"wechatid"` // 微信ID
		StuWxdqDate            sql.NullTime    `db:"stu_wxdq_date"`
		RecipientInfo          sql.NullInt64   `db:"recipient_info"`
		DegreeDepositMoney     sql.NullFloat64 `db:"degree_deposit_money"` // 学位订金
		AgreementId            sql.NullInt64   `db:"agreement_id"`
		StuBir                 sql.NullTime    `db:"stu_bir"` // 学员出生日
		Remark                 sql.NullString  `db:"remark"`
		Plfpbz                 sql.NullString  `db:"plfpbz"`
		StuZcMoney             sql.NullFloat64 `db:"stu_zc_money"` // 暂存总金额
		NewAllotDate           sql.NullTime    `db:"new_allot_date"`
		SchId                  sql.NullInt64   `db:"sch_id"`         // 资源跟进校区ID
		AttendSchool           sql.NullString  `db:"attend_school"`  // 就读学校
		NetworkDetail          sql.NullInt64   `db:"network_detail"` // 渠道细分（NETWORK_DETAIL表）
		StuLevel               sql.NullInt64   `db:"stu_level"`
		OrtherCard             sql.NullString  `db:"orther_card"` // 其他证件号码
		KaoqinTime             sql.NullTime    `db:"kaoqin_time"`
		CreateTime             sql.NullTime    `db:"create_time"`
		Age                    sql.NullInt64   `db:"age"`       // 年龄
		Relation2              sql.NullInt64   `db:"relation2"` // 联系人2关系（数据字典）
		HeadImage              sql.NullInt64   `db:"head_image"`
		StuWxksDate            sql.NullTime    `db:"stu_wxks_date"` // 无效到期日期
		OsName                 sql.NullString  `db:"os_name"`       // 资源收集校区
		TmkCity                sql.NullInt64   `db:"tmk_city"`
		StuLinkPerson          sql.NullString  `db:"stu_link_person"`     // 联系人1
		StudentCustomerId      sql.NullString  `db:"student_customer_id"` // 跟进客服（新增字段）
		IsChildOfEmployee      sql.NullInt64   `db:"is_child_of_employee"`
		StuFollowUpMonthly     sql.NullInt64   `db:"stu_follow_up_monthly"` // 新生跟进月状态（数据字典）
		StuJifen               sql.NullFloat64 `db:"stu_jifen"`             // 学员积分
		EmployeeReferralArea   sql.NullString  `db:"employee_referral_area"`
		UselessRemark          sql.NullString  `db:"useless_remark"`
		Id                     int64           `db:"id"`
		LastUpdateBy           sql.NullString  `db:"last_update_by"`
		StuDjMoney             sql.NullFloat64 `db:"stu_dj_money"` // 定金金额
		AgeGroup               sql.NullInt64   `db:"age_group"`    // 年龄段（数据字典）
		EmployeeReferralDepart sql.NullString  `db:"employee_referral_depart"`
		StuLoadSchool          sql.NullInt64   `db:"stu_load_school"` // 资源收集校区
		Stuschcount            sql.NullFloat64 `db:"stuschcount"`
		StuMydTypeMark         sql.NullString  `db:"stu_myd_type_mark"`
		StuRemark              sql.NullString  `db:"stu_remark"`
		TurnUserType           sql.NullString  `db:"turn_user_type"`
		CrmId                  sql.NullInt64   `db:"crm_id"`
		Relation1              sql.NullInt64   `db:"relation1"` // 联系人1关系（数据字典）
		LoadUser               sql.NullString  `db:"load_user"` // 资源收集人
		TjUser                 sql.NullString  `db:"tj_user"`   // 员工推荐人
		UserLoginState         sql.NullInt64   `db:"user_login_state"`
		StuParentId            sql.NullInt64   `db:"stu_parent_id"`
		QuaoYji                sql.NullInt64   `db:"quao_yji"` // 渠道一级
		Idcard                 sql.NullString  `db:"idcard"`   // 身份证号码
		StuName                sql.NullString  `db:"stu_name"` // 学员姓名
		LastUpdateTime         sql.NullTime    `db:"last_update_time"`
		StuEnName              sql.NullString  `db:"stu_en_name"` // 学员英文
		PkCurrentPay           sql.NullInt64   `db:"pk_current_pay"`
		StuSale                sql.NullString  `db:"stu_sale"`     // 销售
		StuCityId              sql.NullInt64   `db:"stu_city_id"`  // 资源收集城市ID
		IsImportant            sql.NullInt64   `db:"is_important"` // 是否重点(0否，1是）
		StuNowJc               sql.NullInt64   `db:"stu_now_jc"`
		StuAddress             sql.NullString  `db:"stu_address"` // 家庭住址
		ParentsProfession      sql.NullString  `db:"parents_profession"`
		MaTypeRemark           sql.NullString  `db:"ma_type_remark"`   // 来源备注
		KhType                 sql.NullInt64   `db:"kh_type"`          // 客户类型（数据字典）
		PtuserId               sql.NullInt64   `db:"ptuser_id"`        // 兼职人员ID
		NewPlanDate            sql.NullTime    `db:"new_plan_date"`    // 计划回访日期
		MaType                 sql.NullInt64   `db:"ma_type"`          // 来源渠道（活动渠道数据字典）
		StuLinkPerson2         sql.NullString  `db:"stu_link_person2"` // 联系人2
		StuInvalidDate         sql.NullTime    `db:"stu_invalid_date"`
		StuCity                sql.NullString  `db:"stu_city"` // 资源收集城市
		StuMydType             sql.NullString  `db:"stu_myd_type"`
		StuType                sql.NullInt64   `db:"stu_type"`         // 学员类型(数据字典)
		StuPhone2              sql.NullString  `db:"stu_phone2"`       // 联系人2电话
		TimerVisitTime         sql.NullTime    `db:"timer_visit_time"` // 定时回访日期
		StuAllotTmkdate        sql.NullTime    `db:"stu_allot_tmkdate"`
		TmkZxsUsername         sql.NullString  `db:"tmk_zxs_username"`
		IdcardType             sql.NullInt64   `db:"idcard_type"`    // 证件号类型
		RecommendStu           sql.NullInt64   `db:"recommend_stu"`  // 推荐学生ID
		StuSysState            sql.NullInt64   `db:"stu_sys_state"`  // 学员系统状态（数据字典）
		CreateBy               sql.NullString  `db:"create_by"`      // 资源录入人ID
		CallType               sql.NullInt64   `db:"call_type"`      // 咨询类型(数据字典)
		NewVisitDate           sql.NullTime    `db:"new_visit_date"` // 最近回访时间
		ZzxsDate               sql.NullTime    `db:"zzxs_date"`
		HfCs                   sql.NullFloat64 `db:"hf_cs"` // 回访次数
		TmkZxsDate             sql.NullTime    `db:"tmk_zxs_date"`
		StuPhone               sql.NullString  `db:"stu_phone"` // 联系人1电话
		BaobanTime             sql.NullTime    `db:"baoban_time"`
	}
)

func NewStudentModel(conn sqlx.SqlConn, c cache.CacheConf) StudentModel {
	return &defaultStudentModel{
		CachedConn: sqlc.NewConn(conn, c),
		table:      "`student`",
	}
}

func (m *defaultStudentModel) Insert(data Student) (sql.Result, error) {
	query := fmt.Sprintf("insert into %s (%s) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", m.table, studentRowsExpectAutoSet)
	ret, err := m.ExecNoCache(query, data.StuSex, data.DelFlag, data.Wechatid, data.StuWxdqDate, data.RecipientInfo, data.DegreeDepositMoney, data.AgreementId, data.StuBir, data.Remark, data.Plfpbz, data.StuZcMoney, data.NewAllotDate, data.SchId, data.AttendSchool, data.NetworkDetail, data.StuLevel, data.OrtherCard, data.KaoqinTime, data.Age, data.Relation2, data.HeadImage, data.StuWxksDate, data.OsName, data.TmkCity, data.StuLinkPerson, data.StudentCustomerId, data.IsChildOfEmployee, data.StuFollowUpMonthly, data.StuJifen, data.EmployeeReferralArea, data.UselessRemark, data.Id, data.LastUpdateBy, data.StuDjMoney, data.AgeGroup, data.EmployeeReferralDepart, data.StuLoadSchool, data.Stuschcount, data.StuMydTypeMark, data.StuRemark, data.TurnUserType, data.CrmId, data.Relation1, data.LoadUser, data.TjUser, data.UserLoginState, data.StuParentId, data.QuaoYji, data.Idcard, data.StuName, data.LastUpdateTime, data.StuEnName, data.PkCurrentPay, data.StuSale, data.StuCityId, data.IsImportant, data.StuNowJc, data.StuAddress, data.ParentsProfession, data.MaTypeRemark, data.KhType, data.PtuserId, data.NewPlanDate, data.MaType, data.StuLinkPerson2, data.StuInvalidDate, data.StuCity, data.StuMydType, data.StuType, data.StuPhone2, data.TimerVisitTime, data.StuAllotTmkdate, data.TmkZxsUsername, data.IdcardType, data.RecommendStu, data.StuSysState, data.CreateBy, data.CallType, data.NewVisitDate, data.ZzxsDate, data.HfCs, data.TmkZxsDate, data.StuPhone, data.BaobanTime)

	return ret, err
}

func (m *defaultStudentModel) FindOne(id int64) (*Student, error) {
	studentIdKey := fmt.Sprintf("%s%v", cacheStudentIdPrefix, id)
	var resp Student
	err := m.QueryRow(&resp, studentIdKey, func(conn sqlx.SqlConn, v interface{}) error {
		query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", studentRows, m.table)
		return conn.QueryRow(v, query, id)
	})
	switch err {
	case nil:
		return &resp, nil
	case sqlc.ErrNotFound:
		return nil, ErrNotFound
	default:
		return nil, err
	}
}

func (m *defaultStudentModel) Update(data Student) error {
	studentIdKey := fmt.Sprintf("%s%v", cacheStudentIdPrefix, data.Id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("update %s set %s where `id` = ?", m.table, studentRowsWithPlaceHolder)
		return conn.Exec(query, data.StuSex, data.DelFlag, data.Wechatid, data.StuWxdqDate, data.RecipientInfo, data.DegreeDepositMoney, data.AgreementId, data.StuBir, data.Remark, data.Plfpbz, data.StuZcMoney, data.NewAllotDate, data.SchId, data.AttendSchool, data.NetworkDetail, data.StuLevel, data.OrtherCard, data.KaoqinTime, data.Age, data.Relation2, data.HeadImage, data.StuWxksDate, data.OsName, data.TmkCity, data.StuLinkPerson, data.StudentCustomerId, data.IsChildOfEmployee, data.StuFollowUpMonthly, data.StuJifen, data.EmployeeReferralArea, data.UselessRemark, data.LastUpdateBy, data.StuDjMoney, data.AgeGroup, data.EmployeeReferralDepart, data.StuLoadSchool, data.Stuschcount, data.StuMydTypeMark, data.StuRemark, data.TurnUserType, data.CrmId, data.Relation1, data.LoadUser, data.TjUser, data.UserLoginState, data.StuParentId, data.QuaoYji, data.Idcard, data.StuName, data.LastUpdateTime, data.StuEnName, data.PkCurrentPay, data.StuSale, data.StuCityId, data.IsImportant, data.StuNowJc, data.StuAddress, data.ParentsProfession, data.MaTypeRemark, data.KhType, data.PtuserId, data.NewPlanDate, data.MaType, data.StuLinkPerson2, data.StuInvalidDate, data.StuCity, data.StuMydType, data.StuType, data.StuPhone2, data.TimerVisitTime, data.StuAllotTmkdate, data.TmkZxsUsername, data.IdcardType, data.RecommendStu, data.StuSysState, data.CreateBy, data.CallType, data.NewVisitDate, data.ZzxsDate, data.HfCs, data.TmkZxsDate, data.StuPhone, data.BaobanTime, data.Id)
	}, studentIdKey)
	return err
}

func (m *defaultStudentModel) Delete(id int64) error {

	studentIdKey := fmt.Sprintf("%s%v", cacheStudentIdPrefix, id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("delete from %s where `id` = ?", m.table)
		return conn.Exec(query, id)
	}, studentIdKey)
	return err
}

func (m *defaultStudentModel) formatPrimary(primary interface{}) string {
	return fmt.Sprintf("%s%v", cacheStudentIdPrefix, primary)
}

func (m *defaultStudentModel) queryPrimary(conn sqlx.SqlConn, v, primary interface{}) error {
	query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", studentRows, m.table)
	return conn.QueryRow(v, query, primary)
}
