package model

import (
	"database/sql"
	"fmt"
	"strings"
	"time"

	"git.i2edu.net/i2/go-zero/core/stores/cache"
	"git.i2edu.net/i2/go-zero/core/stores/sqlc"
	"git.i2edu.net/i2/go-zero/core/stores/sqlx"
	"git.i2edu.net/i2/go-zero/core/stringx"
	"git.i2edu.net/i2/go-zero/tools/goctl/model/sql/builderx"
)

var (
	sysDataPermissionDetailFieldNames          = builderx.RawFieldNames(&SysDataPermissionDetail{})
	sysDataPermissionDetailRows                = strings.Join(sysDataPermissionDetailFieldNames, ",")
	sysDataPermissionDetailRowsExpectAutoSet   = strings.Join(stringx.Remove(sysDataPermissionDetailFieldNames, "`create_time`", "`update_time`"), ",")
	sysDataPermissionDetailRowsWithPlaceHolder = strings.Join(stringx.Remove(sysDataPermissionDetailFieldNames, "`id`", "`create_time`", "`update_time`"), "=?,") + "=?"

	cacheSysDataPermissionDetailIdPrefix = "cache:sysDataPermissionDetail:id:"
)

type (
	SysDataPermissionDetailModel interface {
		Insert(data SysDataPermissionDetail) (sql.Result, error)
		FindOne(id string) (*SysDataPermissionDetail, error)
		Update(data SysDataPermissionDetail) error
		Delete(id string) error
	}

	defaultSysDataPermissionDetailModel struct {
		sqlc.CachedConn
		table string
	}

	SysDataPermissionDetail struct {
		DelFlag          int64     `db:"del_flag"`
		Id               string    `db:"id"`
		RoleId           string    `db:"role_id"`
		Rule             string    `db:"rule"`
		LastUpdateBy     string    `db:"last_update_by"`
		LastUpdateTime   int64     `db:"last_update_time"`
		DataPermissionId string    `db:"data_permission_id"`
		CreateBy         string    `db:"create_by"`
		CreateTime       time.Time `db:"create_time"`
	}
)

func NewSysDataPermissionDetailModel(conn sqlx.SqlConn, c cache.CacheConf) SysDataPermissionDetailModel {
	return &defaultSysDataPermissionDetailModel{
		CachedConn: sqlc.NewConn(conn, c),
		table:      "`sys_data_permission_detail`",
	}
}

func (m *defaultSysDataPermissionDetailModel) Insert(data SysDataPermissionDetail) (sql.Result, error) {
	query := fmt.Sprintf("insert into %s (%s) values (?, ?, ?, ?, ?, ?, ?, ?)", m.table, sysDataPermissionDetailRowsExpectAutoSet)
	ret, err := m.ExecNoCache(query, data.DelFlag, data.Id, data.RoleId, data.Rule, data.LastUpdateBy, data.LastUpdateTime, data.DataPermissionId, data.CreateBy)

	return ret, err
}

func (m *defaultSysDataPermissionDetailModel) FindOne(id string) (*SysDataPermissionDetail, error) {
	sysDataPermissionDetailIdKey := fmt.Sprintf("%s%v", cacheSysDataPermissionDetailIdPrefix, id)
	var resp SysDataPermissionDetail
	err := m.QueryRow(&resp, sysDataPermissionDetailIdKey, func(conn sqlx.SqlConn, v interface{}) error {
		query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", sysDataPermissionDetailRows, m.table)
		return conn.QueryRow(v, query, id)
	})
	switch err {
	case nil:
		return &resp, nil
	case sqlc.ErrNotFound:
		return nil, ErrNotFound
	default:
		return nil, err
	}
}

func (m *defaultSysDataPermissionDetailModel) Update(data SysDataPermissionDetail) error {
	sysDataPermissionDetailIdKey := fmt.Sprintf("%s%v", cacheSysDataPermissionDetailIdPrefix, data.Id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("update %s set %s where `id` = ?", m.table, sysDataPermissionDetailRowsWithPlaceHolder)
		return conn.Exec(query, data.DelFlag, data.RoleId, data.Rule, data.LastUpdateBy, data.LastUpdateTime, data.DataPermissionId, data.CreateBy, data.Id)
	}, sysDataPermissionDetailIdKey)
	return err
}

func (m *defaultSysDataPermissionDetailModel) Delete(id string) error {

	sysDataPermissionDetailIdKey := fmt.Sprintf("%s%v", cacheSysDataPermissionDetailIdPrefix, id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("delete from %s where `id` = ?", m.table)
		return conn.Exec(query, id)
	}, sysDataPermissionDetailIdKey)
	return err
}

func (m *defaultSysDataPermissionDetailModel) formatPrimary(primary interface{}) string {
	return fmt.Sprintf("%s%v", cacheSysDataPermissionDetailIdPrefix, primary)
}

func (m *defaultSysDataPermissionDetailModel) queryPrimary(conn sqlx.SqlConn, v, primary interface{}) error {
	query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", sysDataPermissionDetailRows, m.table)
	return conn.QueryRow(v, query, primary)
}
