package model

import (
	"database/sql"
	"fmt"
	"strings"
	"time"

	"git.i2edu.net/i2/go-zero/core/stores/sqlc"
	"git.i2edu.net/i2/go-zero/core/stores/sqlx"
	"git.i2edu.net/i2/go-zero/core/stringx"
	"git.i2edu.net/i2/go-zero/tools/goctl/model/sql/builderx"
)

var (
	sysOptionsetFieldNames          = builderx.RawFieldNames(&SysOptionset{})
	sysOptionsetRows                = strings.Join(sysOptionsetFieldNames, ",")
	sysOptionsetRowsExpectAutoSet   = strings.Join(stringx.Remove(sysOptionsetFieldNames, "`create_time`", "`update_time`"), ",")
	sysOptionsetRowsWithPlaceHolder = strings.Join(stringx.Remove(sysOptionsetFieldNames, "`id`", "`create_time`", "`update_time`"), "=?,") + "=?"
)

type (
	SysOptionsetModel interface {
		Insert(data SysOptionset) (sql.Result, error)
		FindOne(id string) (*SysOptionset, error)
		Update(data SysOptionset) error
		Delete(id string) error
	}

	defaultSysOptionsetModel struct {
		conn  sqlx.SqlConn
		table string
	}

	SysOptionset struct {
		DelFlag        int64     `db:"del_flag" json:"del_flag"`
		Name           string    `db:"name" json:"name"`
		Code           string    `db:"code" json:"code"`
		CreateTime     time.Time `db:"create_time" json:"create_time"`
		LastUpdateBy   string    `db:"last_update_by" json:"last_update_by"`
		LastUpdateTime int64     `db:"last_update_time" json:"last_update_time"`
		Id             string    `db:"id" json:"id"`
		Value          string    `db:"value" json:"value"`
		CreateBy       string    `db:"create_by" json:"create_by"`
	}
)

func NewSysOptionsetModel(conn sqlx.SqlConn) SysOptionsetModel {
	return &defaultSysOptionsetModel{
		conn:  conn,
		table: "`sys_optionset`",
	}
}

func (m *defaultSysOptionsetModel) Insert(data SysOptionset) (sql.Result, error) {
	query := fmt.Sprintf("insert into %s (%s) values (?, ?, ?, ?, ?, ?, ?, ?)", m.table, sysOptionsetRowsExpectAutoSet)
	ret, err := m.conn.Exec(query, data.DelFlag, data.Name, data.Code, data.LastUpdateBy, data.LastUpdateTime, data.Id, data.Value, data.CreateBy)
	return ret, err
}

func (m *defaultSysOptionsetModel) FindOne(id string) (*SysOptionset, error) {
	query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", sysOptionsetRows, m.table)
	var resp SysOptionset
	err := m.conn.QueryRow(&resp, query, id)
	switch err {
	case nil:
		return &resp, nil
	case sqlc.ErrNotFound:
		return nil, ErrNotFound
	default:
		return nil, err
	}
}

func (m *defaultSysOptionsetModel) Update(data SysOptionset) error {
	query := fmt.Sprintf("update %s set %s where `id` = ?", m.table, sysOptionsetRowsWithPlaceHolder)
	_, err := m.conn.Exec(query, data.DelFlag, data.Name, data.Code, data.LastUpdateBy, data.LastUpdateTime, data.Value, data.CreateBy, data.Id)
	return err
}

func (m *defaultSysOptionsetModel) Delete(id string) error {
	query := fmt.Sprintf("delete from %s where `id` = ?", m.table)
	_, err := m.conn.Exec(query, id)
	return err
}
