package model

import (
	"database/sql"
	"fmt"
	"strings"
	"time"

	"git.i2edu.net/i2/go-zero/core/stores/cache"
	"git.i2edu.net/i2/go-zero/core/stores/sqlc"
	"git.i2edu.net/i2/go-zero/core/stores/sqlx"
	"git.i2edu.net/i2/go-zero/core/stringx"
	"git.i2edu.net/i2/go-zero/tools/goctl/model/sql/builderx"
)

var (
	sysRoleDataPermissionFieldNames          = builderx.RawFieldNames(&SysRoleDataPermission{})
	sysRoleDataPermissionRows                = strings.Join(sysRoleDataPermissionFieldNames, ",")
	sysRoleDataPermissionRowsExpectAutoSet   = strings.Join(stringx.Remove(sysRoleDataPermissionFieldNames, "`create_time`", "`update_time`"), ",")
	sysRoleDataPermissionRowsWithPlaceHolder = strings.Join(stringx.Remove(sysRoleDataPermissionFieldNames, "`id`", "`create_time`", "`update_time`"), "=?,") + "=?"

	cacheSysRoleDataPermissionIdPrefix = "cache:sysRoleDataPermission:id:"
)

type (
	SysRoleDataPermissionModel interface {
		Insert(data SysRoleDataPermission) (sql.Result, error)
		FindOne(id string) (*SysRoleDataPermission, error)
		Update(data SysRoleDataPermission) error
		Delete(id string) error
	}

	defaultSysRoleDataPermissionModel struct {
		sqlc.CachedConn
		table string
	}

	SysRoleDataPermission struct {
		RoleId           string    `db:"role_id"`
		CreateBy         string    `db:"create_by"`
		CreateTime       time.Time `db:"create_time"`
		LastUpdateBy     string    `db:"last_update_by"`
		LastUpdateTime   int64     `db:"last_update_time"`
		DelFlag          int64     `db:"del_flag"`
		Id               string    `db:"id"`
		DataPermissionId string    `db:"data_permission_id"`
	}
)

func NewSysRoleDataPermissionModel(conn sqlx.SqlConn, c cache.CacheConf) SysRoleDataPermissionModel {
	return &defaultSysRoleDataPermissionModel{
		CachedConn: sqlc.NewConn(conn, c),
		table:      "`sys_role_data_permission`",
	}
}

func (m *defaultSysRoleDataPermissionModel) Insert(data SysRoleDataPermission) (sql.Result, error) {
	query := fmt.Sprintf("insert into %s (%s) values (?, ?, ?, ?, ?, ?, ?)", m.table, sysRoleDataPermissionRowsExpectAutoSet)
	ret, err := m.ExecNoCache(query, data.RoleId, data.CreateBy, data.LastUpdateBy, data.LastUpdateTime, data.DelFlag, data.Id, data.DataPermissionId)

	return ret, err
}

func (m *defaultSysRoleDataPermissionModel) FindOne(id string) (*SysRoleDataPermission, error) {
	sysRoleDataPermissionIdKey := fmt.Sprintf("%s%v", cacheSysRoleDataPermissionIdPrefix, id)
	var resp SysRoleDataPermission
	err := m.QueryRow(&resp, sysRoleDataPermissionIdKey, func(conn sqlx.SqlConn, v interface{}) error {
		query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", sysRoleDataPermissionRows, m.table)
		return conn.QueryRow(v, query, id)
	})
	switch err {
	case nil:
		return &resp, nil
	case sqlc.ErrNotFound:
		return nil, ErrNotFound
	default:
		return nil, err
	}
}

func (m *defaultSysRoleDataPermissionModel) Update(data SysRoleDataPermission) error {
	sysRoleDataPermissionIdKey := fmt.Sprintf("%s%v", cacheSysRoleDataPermissionIdPrefix, data.Id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("update %s set %s where `id` = ?", m.table, sysRoleDataPermissionRowsWithPlaceHolder)
		return conn.Exec(query, data.RoleId, data.CreateBy, data.LastUpdateBy, data.LastUpdateTime, data.DelFlag, data.DataPermissionId, data.Id)
	}, sysRoleDataPermissionIdKey)
	return err
}

func (m *defaultSysRoleDataPermissionModel) Delete(id string) error {

	sysRoleDataPermissionIdKey := fmt.Sprintf("%s%v", cacheSysRoleDataPermissionIdPrefix, id)
	_, err := m.Exec(func(conn sqlx.SqlConn) (result sql.Result, err error) {
		query := fmt.Sprintf("delete from %s where `id` = ?", m.table)
		return conn.Exec(query, id)
	}, sysRoleDataPermissionIdKey)
	return err
}

func (m *defaultSysRoleDataPermissionModel) formatPrimary(primary interface{}) string {
	return fmt.Sprintf("%s%v", cacheSysRoleDataPermissionIdPrefix, primary)
}

func (m *defaultSysRoleDataPermissionModel) queryPrimary(conn sqlx.SqlConn, v, primary interface{}) error {
	query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", sysRoleDataPermissionRows, m.table)
	return conn.QueryRow(v, query, primary)
}
